package eu.ztsh.lib.time;

/**
 * Structure for summarized work time.
 *
 * @param summary Short description
 * @param time    String representation of spent hours
 * @param diff    Difference between expected and real work time as {@link Diff} object
 */
public record Calculation(String summary, String time, Diff diff) {

    public static Calculation calculate(String summary, long daysTillToday, long secondsSpent) {
        return new Calculation(summary,
                String.valueOf(((double) secondsSpent) / (60 * 60)),
                Diff.calculate(daysTillToday, secondsSpent));
    }

    public static Calculation off(String summary) {
        return new Calculation(summary, "0.0", Diff.OFF);
    }

}
