package eu.ztsh.lib.time;

import org.junit.jupiter.api.Test;

import java.time.LocalDateTime;
import java.time.Month;
import java.util.TreeSet;

import static org.assertj.core.api.Assertions.assertThat;

class DurationTest {

    private final LocalDateTime base = LocalDateTime.of(2023, Month.JULY, 3, 8, 45);
    private final Duration nullDuration = new Duration(base, 0, base);
    private final Duration longDuration = new Duration(base, 3 * 60 * 60, base.plusHours(3));
    private final Duration shortDuration = new Duration(base, 30 * 60, base.plusMinutes(30));

    @Test
    public void betweenSimpleComputeTest() {
        var next = base.plusSeconds(100);

        var previousDuration = new Duration(base, 0, base);
        var nextDuration = new Duration(next, 0, next);

        assertThat(Duration.between(previousDuration, nextDuration).getTimeSpentSeconds()).isEqualTo(100);
    }

    @Test
    public void betweenInboundComputeTest() {
        assertThat(Duration.between(longDuration, shortDuration)).isEqualByComparingTo(nullDuration);
        assertThat(Duration.between(shortDuration, longDuration)).isEqualByComparingTo(nullDuration);
    }

    @Test
    public void sortingTest() {
        var set = new TreeSet<Duration>();
        set.add(nullDuration);
        set.add(shortDuration);
        set.add(longDuration);
        assertThat(set).containsExactly(longDuration, shortDuration, nullDuration);
    }

}
