package eu.ztsh.lib.time.util;

import org.junit.jupiter.api.Test;

import java.time.LocalDate;

import static eu.ztsh.lib.time.util.TimeUtil.countUntil;
import static org.assertj.core.api.Assertions.assertThat;

class TimeUtilTest {

    @Test
    public void secondMondayTest() {
        assertThat(TimeUtil.isMonthBeginning(LocalDate.of(2023, 5, 8))).isFalse();
    }

    @Test
    public void firstSundayTest() {
        assertThat(TimeUtil.isMonthBeginning(LocalDate.of(2023, 5, 7))).isTrue();
    }

    @Test
    public void firstTuesdayTest() {
        assertThat(TimeUtil.isMonthBeginning(LocalDate.of(2023, 5, 2))).isTrue();
    }

    @Test
    public void firstMondayTest() {
        assertThat(TimeUtil.isMonthBeginning(LocalDate.of(2022, 11, 7))).isTrue();
    }

    @Test
    public void countTest() {
        var today = LocalDate.of(2023, 7, 23); // Sunday
        var last = TimeUtil.getLastDayOfMonth(today);
        var ranges = TimeRangeUtil.createRanges(last);
        assertThat(ranges).hasSize(6);
        assertThat(countUntil(ranges, today)).isEqualTo(15);
        assertThat(countUntil(ranges, today.plusDays(1))) // Monday
                .isEqualTo(16);
    }

    @Test
    public void mondayCalculationTest() {
        var monday = LocalDate.of(2023, 5, 1);
        assertThat(TimeUtil.getLastMonday(monday)).isEqualTo(monday);
        assertThat(TimeUtil.getLastMonday(monday.plusDays(2))).isEqualTo(monday);
    }

}
